const express = require('express');

const enrollmentApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`);

const response = require(`${libs}/middlewares/response`);

const parseParams = require(`${libs}/middlewares/parseParams`);

// **Temporary** solution to add where clauses that are common to all requests

// Complete range of the enrollments dataset.
// Returns a tuple of start and ending years of the complete enrollments dataset.
enrollmentApp.get('/year_range', (req, res, next) => {
    req.sql = squel.select()
        .from('turma')
        .field('MIN(turma.ano_censo)', 'start_year')
        .field('MAX(turma.ano_censo)', 'end_year');

    next();
}, query, response('range'));

// Returns all educational levels avaible
enrollmentApp.get('/education_level', (req, res, next) => {
    req.sql = squel.select()
        .from('etapa_ensino')
        .field('pk_etapa_ensino_id', 'id')
        .field('desc_etapa', 'name');

    next();
}, query, response('education_level'));

// Returns all adm dependencies
enrollmentApp.get('/adm_dependency', (req, res, next) => {
    req.sql = squel.select()
        .from('dependencia_adm')
        .field('pk_dependencia_adm_id', 'id')
        .field('nome', 'name');

    next();
}, query, response('adm_dependency'));

// Parse the filters and dimensions parameter in the query
enrollmentApp.use('/', parseParams('filter', [
    'min_year',
    'max_year',
    'adm_dependency',
    'location',
    'education_level',
    'region',
    'state',
    'city',
    'school'
]), parseParams('dims', [
    'adm_dependency',
    'location',
    'education_level',
    'region',
    'state',
    'city',
    'school'
]), (req, res, next) => {
    log.debug(req.filter);
    log.debug(req.dims);

    //applyJoins(req.sql, req.filter, req.dims);

    // Do the joins
    if(typeof req.filter.adm_dependency !== 'undefined'
        || typeof req.dims.adm_dependency !== 'undefined') {
        req.sql.join('dependencia_adm', null,
            'fk_dependencia_adm_id = dependencia_adm.pk_dependencia_adm_id');
    }

    if(typeof req.filter.education_level !== 'undefined'
        || typeof req.dims.education_level !== 'undefined') {
        req.sql.join('etapa_ensino', null,
            'fk_etapa_ensino_id = etapa_ensino.pk_etapa_ensino_id');
    }

    if(typeof req.filter.region !== 'undefined'
        || typeof req.dims.region !== 'undefined') {
            req.sql.join('municipio', null, 'fk_municipio_id = municipio.pk_cod_ibge')
                .join('estado', null, 'municipio.fk_estado_id = estado.pk_estado_id')
                .join('regiao', null, 'estado.fk_regiao_id = regiao.pk_regiao_id');
    }

    if((typeof req.filter.state !== 'undefined'
        || typeof req.dims.state !== 'undefined')
        && (typeof req.filter.region === 'undefined'
        && typeof req.dims.region === 'undefined')) {
            req.sql.join('municipio', null, 'fk_municipio_id = municipio.pk_cod_ibge')
                .join('estado', null, 'municipio.fk_estado_id = estado.pk_estado_id');
    }

    if((typeof req.filter.city !== 'undefined'
        || typeof req.dims.city !== 'undefined')
        && (typeof req.filter.state === 'undefined'
        && typeof req.dims.state === 'undefined')
        && (typeof req.filter.region === 'undefined'
        && typeof req.dims.region === 'undefined')) {
        req.sql.join('municipio', null, 'fk_municipio_id = municipio.pk_cod_ibge');
    }

    if(typeof req.dims.school !== 'undefined') {
        req.sql.join('escola', null, 'turma.cod_entidade = escola.cod_entidade');
    }

    // Dimensions (add fields)

    if(typeof req.dims.education_level !== 'undefined') {
        req.sql.field('desc_etapa', 'education_level')
            .group('desc_etapa')
            .order('desc_etapa');
    }

    if(typeof req.dims.region !== 'undefined') {
        req.sql.field('regiao.nome', 'region_name')
            .group('regiao.nome')
            .order('regiao.nome');
    }

    if(typeof req.dims.state !== 'undefined') {
        req.sql.field('estado.nome', 'state_name')
            .group('estado.nome')
            .order('estado.nome');
    }

    if(typeof req.dims.city !== 'undefined') {
        req.sql.field('municipio.nome', 'city_name')
            .group('municipio.nome')
            .order('municipio.nome');
    }

    /*
    if(typeof req.dims.school !== 'undefined') {
        req.sql.field('escolas.nome_entidade', 'school_name')
            .group('escolas.nome_entidade')
            .order('escolas.nome_entidade');
    }
    */

    if(typeof req.dims.adm_dependency !== 'undefined') {
        req.sql.field('dependencia_adm.nome', 'adm_dependency_name')
            .group('dependencia_adm.nome')
            .order('dependencia_adm.nome');
    }

    if(typeof req.dims.location !== 'undefined') {
        req.sql.field('turma.id_localizacao', 'location_name')
            .group('turma.id_localizacao')
            .order('turma.id_localizacao');
    }

    if(typeof req.dims.region === 'undefined'
        && typeof req.dims.state === 'undefined'
        && typeof req.dims.city === 'undefined'
        && typeof req.dims.school === 'undefined') {
        req.sql.field("'Brasil'", 'name');
    }

    // Filter (add where)

    if (typeof req.filter.min_year !== 'undefined') {
        req.sql.where('turma.ano_censo >= ?', parseInt(req.filter.min_year, 10));
    }

    if (typeof req.filter.max_year !== 'undefined') {
        req.sql.where('turma.ano_censo <= ?', parseInt(req.filter.max_year, 10));
    }

    if (typeof req.filter.adm_dependency !== 'undefined') {
        req.sql.where('pk_dependencia_adm_id = ?', parseInt(req.filter.adm_dependency, 10));
    }

    if (typeof req.filter.location !== 'undefined') {
        req.sql.where('turma.id_localizacao = ?', parseInt(req.filter.location, 10));
    }

    if (typeof req.filter.education_level !== 'undefined') {
        req.sql.where('pk_etapa_ensino_id = ?', parseInt(req.filter.education_level, 10));
    }

    if (typeof req.filter.region !== 'undefined') {
        req.sql.where('pk_regiao_id = ?', parseInt(req.filter.region, 10));
    }

    if (typeof req.filter.state !== 'undefined') {
        req.sql.where('pk_estado_id = ?', parseInt(req.filter.state, 10));
    }

    if (typeof req.filter.city !== 'undefined') {
        req.sql.where('turma.fk_municipio_id = ?', parseInt(req.filter.city, 10));
    }

    if (typeof req.filter.school !== 'undefined') {
        req.sql.where('turma.fk_escola_id = ?', parseInt(req.filter.school, 10));
    }
    log.debug(req.sql.toParam());
    next();
});

enrollmentApp.get('/', (req, res, next) => {
    req.sql.field('COALESCE(SUM(num_matriculas), 0)', 'total')
        .field('turma.ano_censo', 'year')
        .from('turma')
        .group('turma.ano_censo')
        .order('turma.ano_censo');
    next();
}, query, (req, res, next) => {
    // 'Sanitize' result
    let r = req.result;
    r.forEach((data) => {
        if(req.dims.location) {
            if(data.location_name === 1) {
                data.location_name = 'Urbana';
            } else {
                data.location_name = 'Rural';
            }
        }
    });
    next();
}, response('enrollment'));

module.exports = enrollmentApp;
